=== modified file 'drizzled/Makefile.am'
--- drizzled/Makefile.am	2009-12-05 23:29:59 +0000
+++ drizzled/Makefile.am	2009-12-07 21:50:09 +0000
@@ -257,6 +257,9 @@
 		optimizer/key_field.h \
 		optimizer/key_use.h \
 		optimizer/position.h \
+		optimizer/quick_index_merge_select.h \
+		optimizer/quick_range.h \
+		optimizer/quick_range_select.h \
 		optimizer/range.h \
 		optimizer/sargable_param.h \
 		optimizer/sum.h \
@@ -606,6 +609,9 @@
 		name_resolution_context_state.cc \
 		natural_join_column.cc \
 		optimizer/key_field.cc \
+		optimizer/quick_index_merge_select.cc \
+		optimizer/quick_range.cc \
+		optimizer/quick_range_select.cc \
 		optimizer/range.cc \
 		optimizer/sum.cc \
 		plugin/authentication.cc \

=== modified file 'drizzled/filesort.cc'
--- drizzled/filesort.cc	2009-12-07 19:18:46 +0000
+++ drizzled/filesort.cc	2009-12-07 21:50:09 +0000
@@ -134,7 +134,7 @@
 
   /*
     Don't use table->sort in filesort as it is also used by
-    QUICK_INDEX_MERGE_SELECT. Work with a copy and put it back at the end
+    QuickIndexMergeSelect. Work with a copy and put it back at the end
     when index_merge select has finished with it.
   */
   memcpy(&table_sort, &table->sort, sizeof(filesort_info_st));

=== added file 'drizzled/optimizer/quick_index_merge_select.cc'
--- drizzled/optimizer/quick_index_merge_select.cc	1970-01-01 00:00:00 +0000
+++ drizzled/optimizer/quick_index_merge_select.cc	2009-12-07 21:50:09 +0000
@@ -0,0 +1,267 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008-2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/records.h"
+#include "drizzled/optimizer/quick_range_select.h"
+#include "drizzled/optimizer/quick_index_merge_select.h"
+
+using namespace drizzled;
+
+static int refpos_order_cmp(void *arg, const void *a, const void *b)
+{
+  Cursor *cursor= (Cursor*)arg;
+  return cursor->cmp_ref((const unsigned char *) a, (const unsigned char *) b);
+}
+
+optimizer::QuickIndexMergeSelect::QuickIndexMergeSelect(Session *session_param,
+                                                        Table *table)
+  :
+    pk_quick_select(NULL), 
+    session(session_param)
+{
+  index= MAX_KEY;
+  head= table;
+  memset(&read_record, 0, sizeof(read_record));
+  init_sql_alloc(&alloc, session->variables.range_alloc_block_size, 0);
+  return;
+}
+
+int optimizer::QuickIndexMergeSelect::init()
+{
+  return 0;
+}
+
+int optimizer::QuickIndexMergeSelect::reset()
+{
+  return (read_keys_and_merge());
+}
+
+bool
+optimizer::QuickIndexMergeSelect::push_quick_back(optimizer::QuickRangeSelect *quick_sel_range)
+{
+  /*
+    Save quick_select that does scan on clustered primary key as it will be
+    processed separately.
+  */
+  if (head->cursor->primary_key_is_clustered() &&
+      quick_sel_range->index == head->s->primary_key)
+  {
+    pk_quick_select= quick_sel_range;
+  }
+  else
+  {
+    return quick_selects.push_back(quick_sel_range);
+  }
+  return 0;
+}
+
+optimizer::QuickIndexMergeSelect::~QuickIndexMergeSelect()
+{
+  List_iterator_fast<optimizer::QuickRangeSelect> quick_it(quick_selects);
+  optimizer::QuickRangeSelect* quick;
+  quick_it.rewind();
+  while ((quick= quick_it++))
+  {
+    quick->cursor= NULL;
+  }
+  quick_selects.delete_elements();
+  delete pk_quick_select;
+  free_root(&alloc,MYF(0));
+  return;
+}
+
+
+int optimizer::QuickIndexMergeSelect::read_keys_and_merge()
+{
+  List_iterator_fast<optimizer::QuickRangeSelect> cur_quick_it(quick_selects);
+  optimizer::QuickRangeSelect* cur_quick;
+  int result;
+  Unique *unique;
+  Cursor *cursor= head->cursor;
+
+  cursor->extra(HA_EXTRA_KEYREAD);
+  head->prepare_for_position();
+
+  cur_quick_it.rewind();
+  cur_quick= cur_quick_it++;
+  assert(cur_quick != 0);
+
+  /*
+    We reuse the same instance of Cursor so we need to call both init and
+    reset here.
+  */
+  if (cur_quick->init() || cur_quick->reset())
+    return 0;
+
+  unique= new Unique(refpos_order_cmp, 
+                     (void *)cursor,
+                     cursor->ref_length,
+                     session->variables.sortbuff_size);
+  if (!unique)
+    return 0;
+  for (;;)
+  {
+    while ((result= cur_quick->get_next()) == HA_ERR_END_OF_FILE)
+    {
+      cur_quick->range_end();
+      cur_quick= cur_quick_it++;
+      if (!cur_quick)
+        break;
+
+      if (cur_quick->cursor->inited != Cursor::NONE)
+        cur_quick->cursor->ha_index_end();
+      if (cur_quick->init() || cur_quick->reset())
+        return 0;
+    }
+
+    if (result)
+    {
+      if (result != HA_ERR_END_OF_FILE)
+      {
+        cur_quick->range_end();
+        return result;
+      }
+      break;
+    }
+
+    if (session->killed)
+      return 0;
+
+    /* skip row if it will be retrieved by clustered PK scan */
+    if (pk_quick_select && pk_quick_select->row_in_ranges())
+      continue;
+
+    cur_quick->cursor->position(cur_quick->record);
+    result= unique->unique_add((char*)cur_quick->cursor->ref);
+    if (result)
+      return 0;
+
+  }
+
+  /* ok, all row ids are in Unique */
+  result= unique->get(head);
+  delete unique;
+  doing_pk_scan= false;
+  /* index_merge currently doesn't support "using index" at all */
+  cursor->extra(HA_EXTRA_NO_KEYREAD);
+  /* start table scan */
+  init_read_record(&read_record, session, head, (optimizer::SqlSelect*) 0, 1, 1);
+  return result;
+}
+
+
+int optimizer::QuickIndexMergeSelect::get_next()
+{
+  int result;
+
+  if (doing_pk_scan)
+    return(pk_quick_select->get_next());
+
+  if ((result= read_record.read_record(&read_record)) == -1)
+  {
+    result= HA_ERR_END_OF_FILE;
+    end_read_record(&read_record);
+    /* All rows from Unique have been retrieved, do a clustered PK scan */
+    if (pk_quick_select)
+    {
+      doing_pk_scan= true;
+      if ((result= pk_quick_select->init()) ||
+          (result= pk_quick_select->reset()))
+        return result;
+      return(pk_quick_select->get_next());
+    }
+  }
+
+  return result;
+}
+
+bool optimizer::QuickIndexMergeSelect::is_keys_used(const MyBitmap *fields)
+{
+  optimizer::QuickRangeSelect *quick= NULL;
+  List_iterator_fast<QuickRangeSelect> it(quick_selects);
+  while ((quick= it++))
+  {
+    if (is_key_used(head, quick->index, fields))
+      return 1;
+  }
+  return 0;
+}
+
+
+void optimizer::QuickIndexMergeSelect::add_info_string(String *str)
+{
+  optimizer::QuickRangeSelect *quick= NULL;
+  bool first= true;
+  List_iterator_fast<optimizer::QuickRangeSelect> it(quick_selects);
+  str->append(STRING_WITH_LEN("sort_union("));
+  while ((quick= it++))
+  {
+    if (! first)
+      str->append(',');
+    else
+      first= false;
+    quick->add_info_string(str);
+  }
+  if (pk_quick_select)
+  {
+    str->append(',');
+    pk_quick_select->add_info_string(str);
+  }
+  str->append(')');
+}
+
+
+void optimizer::QuickIndexMergeSelect::add_keys_and_lengths(String *key_names,
+                                                            String *used_lengths)
+{
+  char buf[64];
+  uint32_t length;
+  bool first= true;
+  optimizer::QuickRangeSelect *quick= NULL;
+
+  List_iterator_fast<optimizer::QuickRangeSelect> it(quick_selects);
+  while ((quick= it++))
+  {
+    if (first)
+      first= false;
+    else
+    {
+      key_names->append(',');
+      used_lengths->append(',');
+    }
+
+    KEY *key_info= head->key_info + quick->index;
+    key_names->append(key_info->name);
+    length= int64_t2str(quick->max_used_key_length, buf, 10) - buf;
+    used_lengths->append(buf, length);
+  }
+  if (pk_quick_select)
+  {
+    KEY *key_info= head->key_info + pk_quick_select->index;
+    key_names->append(',');
+    key_names->append(key_info->name);
+    length= int64_t2str(pk_quick_select->max_used_key_length, buf, 10) - buf;
+    used_lengths->append(',');
+    used_lengths->append(buf, length);
+  }
+}
+
+

=== added file 'drizzled/optimizer/quick_index_merge_select.h'
--- drizzled/optimizer/quick_index_merge_select.h	1970-01-01 00:00:00 +0000
+++ drizzled/optimizer/quick_index_merge_select.h	2009-12-07 21:50:09 +0000
@@ -0,0 +1,169 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008-2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_OPTIMIZER_QuickIndexMergeSelect_H
+#define DRIZZLED_OPTIMIZER_QuickIndexMergeSelect_H
+
+#include "drizzled/optimizer/range.h"
+
+namespace drizzled
+{
+
+namespace optimizer
+{
+
+/**
+  @class QuickIndexMergeSelect - index_merge access method quick select.
+
+    QuickIndexMergeSelect uses
+     * QuickRangeSelects to get rows
+     * Unique class to remove duplicate rows
+
+  INDEX MERGE OPTIMIZER
+    Current implementation doesn't detect all cases where index_merge could
+    be used, in particular:
+     * index_merge will never be used if range scan is possible (even if
+       range scan is more expensive)
+
+     * index_merge+'using index' is not supported (this the consequence of
+       the above restriction)
+
+     * If WHERE part contains complex nested AND and OR conditions, some ways
+       to retrieve rows using index_merge will not be considered. The choice
+       of read plan may depend on the order of conjuncts/disjuncts in WHERE
+       part of the query, see comments near imerge_list_or_list and
+       SEL_IMERGE::or_sel_tree_with_checks functions for details.
+
+     * There is no "index_merge_ref" method (but index_merge on non-first
+       table in join is possible with 'range checked for each record').
+
+    See comments around SEL_IMERGE class and test_quick_select for more
+    details.
+
+  ROW RETRIEVAL ALGORITHM
+
+    index_merge uses Unique class for duplicates removal.  index_merge takes
+    advantage of Clustered Primary Key (CPK) if the table has one.
+    The index_merge algorithm consists of two phases:
+
+    Phase 1 (implemented in QuickIndexMergeSelect::prepare_unique):
+    prepare()
+    {
+      activate 'index only';
+      while(retrieve next row for non-CPK scan)
+      {
+        if (there is a CPK scan and row will be retrieved by it)
+          skip this row;
+        else
+          put its rowid into Unique;
+      }
+      deactivate 'index only';
+    }
+
+    Phase 2 (implemented as sequence of QuickIndexMergeSelect::get_next
+    calls):
+
+    fetch()
+    {
+      retrieve all rows from row pointers stored in Unique;
+      free Unique;
+      retrieve all rows for CPK scan;
+    }
+*/
+class QuickIndexMergeSelect : public QuickSelectInterface
+{
+public:
+
+  QuickIndexMergeSelect(Session *session, Table *table);
+
+  ~QuickIndexMergeSelect();
+
+  int init();
+  int reset(void);
+
+  /**
+   * Get next row for index_merge.
+   * NOTES
+   * The rows are read from
+   * 1. rowids stored in Unique.
+   * 2. QuickRangeSelect with clustered primary key (if any).
+   * The sets of rows retrieved in 1) and 2) are guaranteed to be disjoint.
+   */
+  int get_next();
+
+  bool reverse_sorted()
+  {
+    return false;
+  }
+
+  bool unique_key_range()
+  {
+    return false;
+  }
+
+  int get_type()
+  {
+    return QS_TYPE_INDEX_MERGE;
+  }
+
+  void add_keys_and_lengths(String *key_names, String *used_lengths);
+  void add_info_string(String *str);
+  bool is_keys_used(const MyBitmap *fields);
+
+  bool push_quick_back(QuickRangeSelect *quick_sel_range);
+
+  /* range quick selects this index_merge read consists of */
+  List<QuickRangeSelect> quick_selects;
+
+  /* quick select that uses clustered primary key (NULL if none) */
+  QuickRangeSelect* pk_quick_select;
+
+  /* true if this select is currently doing a clustered PK scan */
+  bool  doing_pk_scan;
+
+  MEM_ROOT alloc;
+  Session *session;
+
+  /**
+   * Perform key scans for all used indexes (except CPK), get rowids and merge
+   * them into an ordered non-recurrent sequence of rowids.
+   *
+   * The merge/duplicate removal is performed using Unique class. We put all
+   * rowids into Unique, get the sorted sequence and destroy the Unique.
+   *
+   * If table has a clustered primary key that covers all rows (true for bdb
+   * and innodb currently) and one of the index_merge scans is a scan on PK,
+   * then rows that will be retrieved by PK scan are not put into Unique and
+   * primary key scan is not performed here, it is performed later separately.
+   *
+   * RETURN
+   * @retval 0     OK
+   * @retval other error
+   */
+  int read_keys_and_merge();
+
+  /* used to get rows collected in Unique */
+  READ_RECORD read_record;
+};
+
+} /* namespace optimizer */
+
+} /* namespace drizzled */
+
+#endif /* DRIZZLED_OPTIMIZER_QuickIndexMergeSelect_H */

=== added file 'drizzled/optimizer/quick_range.cc'
--- drizzled/optimizer/quick_range.cc	1970-01-01 00:00:00 +0000
+++ drizzled/optimizer/quick_range.cc	2009-12-07 21:50:09 +0000
@@ -0,0 +1,34 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008-2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/optimizer/quick_range.h"
+
+using namespace drizzled;
+
+optimizer::QuickRange::QuickRange()
+  :
+    min_key(0),
+    max_key(0),
+    min_length(0),
+    max_length(0),
+    flag(NO_MIN_RANGE | NO_MAX_RANGE),
+    min_keypart_map(0), 
+    max_keypart_map(0)
+{}

=== added file 'drizzled/optimizer/quick_range.h'
--- drizzled/optimizer/quick_range.h	1970-01-01 00:00:00 +0000
+++ drizzled/optimizer/quick_range.h	2009-12-07 21:50:09 +0000
@@ -0,0 +1,63 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008-2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_OPTIMIZER_QUICK_RANGE_H
+#define DRIZZLED_OPTIMIZER_QUICK_RANGE_H
+
+namespace drizzled
+{
+
+namespace optimizer
+{
+
+class QuickRange : public Sql_alloc 
+{
+public:
+  unsigned char *min_key;
+  unsigned char *max_key;
+  uint16_t min_length;
+  uint16_t max_length;
+  uint16_t flag;
+  key_part_map min_keypart_map; /**< bitmap of used keyparts in min_key */
+  key_part_map max_keypart_map; /**< bitmap of used keyparts in max_key */
+
+  QuickRange(); /**< Constructor for a "full range" */
+  QuickRange(const unsigned char *min_key_arg,
+              uint32_t min_length_arg,
+              key_part_map min_keypart_map_arg,
+	            const unsigned char *max_key_arg, 
+              uint32_t max_length_arg,
+              key_part_map max_keypart_map_arg,
+	            uint32_t flag_arg)
+    : 
+      min_key((unsigned char*) sql_memdup(min_key_arg,min_length_arg+1)),
+      max_key((unsigned char*) sql_memdup(max_key_arg,max_length_arg+1)),
+      min_length((uint16_t) min_length_arg),
+      max_length((uint16_t) max_length_arg),
+      flag((uint16_t) flag_arg),
+      min_keypart_map(min_keypart_map_arg),
+      max_keypart_map(max_keypart_map_arg)
+    {}
+};
+
+} /* namespace optimizer */
+
+} /* namespace drizzled */
+
+#endif /* DRIZZLED_OPTIMIZER_QUICK_RANGE_H */

=== added file 'drizzled/optimizer/quick_range_select.cc'
--- drizzled/optimizer/quick_range_select.cc	1970-01-01 00:00:00 +0000
+++ drizzled/optimizer/quick_range_select.cc	2009-12-07 21:50:09 +0000
@@ -0,0 +1,615 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008-2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/optimizer/quick_range.h"
+#include "drizzled/optimizer/quick_range_select.h"
+#include "mysys/my_bitmap.h"
+#include "drizzled/memory/multi_malloc.h"
+
+using namespace std;
+using namespace drizzled;
+
+
+optimizer::QuickRangeSelect::QuickRangeSelect(Session *session, 
+                                              Table *table, 
+                                              uint32_t key_nr,
+                                              bool no_alloc, 
+                                              MEM_ROOT *parent_alloc,
+                                              bool *create_error)
+  :
+    cursor(NULL),
+    ranges(),
+    in_ror_merged_scan(false),
+    column_bitmap(),
+    save_read_set(NULL),
+    save_write_set(NULL),
+    free_file(false),
+    cur_range(NULL),
+    last_range(NULL),
+    qr_traversal_ctx(),
+    mrr_buf_size(0),
+    mrr_buf_desc(NULL),
+    key_parts(NULL),
+    dont_free(false),
+    mrr_flags(0),
+    alloc()
+{
+  my_bitmap_map *bitmap= NULL;
+
+  sorted= 0;
+  index= key_nr;
+  head= table;
+  key_part_info= head->key_info[index].key_part;
+  my_init_dynamic_array(&ranges, sizeof(optimizer::QuickRange*), 16, 16);
+
+  /* 'session' is not accessible in QuickRangeSelect::reset(). */
+  mrr_buf_size= session->variables.read_rnd_buff_size;
+  mrr_buf_desc= NULL;
+
+  if (! no_alloc && ! parent_alloc)
+  {
+    // Allocates everything through the internal memroot
+    init_sql_alloc(&alloc, session->variables.range_alloc_block_size, 0);
+    session->mem_root= &alloc;
+  }
+  else
+  {
+    memset(&alloc, 0, sizeof(alloc));
+  }
+
+  cursor= head->cursor;
+  record= head->record[0];
+  save_read_set= head->read_set;
+  save_write_set= head->write_set;
+
+  /* Allocate a bitmap for used columns. Using sql_alloc instead of malloc
+     simply as a "fix" to the MySQL 6.0 code that also free()s it at the
+     same time we destroy the mem_root.
+   */
+
+  bitmap= reinterpret_cast<my_bitmap_map*>(sql_alloc(head->s->column_bitmap_size));
+  if (! bitmap)
+  {
+    column_bitmap.setBitmap(NULL);
+    *create_error= 1;
+  }
+  else
+  {
+    column_bitmap.init(bitmap, head->s->fields);
+  }
+}
+
+
+int optimizer::QuickRangeSelect::init()
+{
+  if (cursor->inited != Cursor::NONE)
+    cursor->ha_index_or_rnd_end();
+  return (cursor->ha_index_init(index, 1));
+}
+
+
+void optimizer::QuickRangeSelect::range_end()
+{
+  if (cursor->inited != Cursor::NONE)
+    cursor->ha_index_or_rnd_end();
+}
+
+
+optimizer::QuickRangeSelect::~QuickRangeSelect()
+{
+  if (! dont_free)
+  {
+    /* cursor is NULL for CPK scan on covering ROR-intersection */
+    if (cursor)
+    {
+      range_end();
+      if (head->key_read)
+      {
+        head->key_read= 0;
+        cursor->extra(HA_EXTRA_NO_KEYREAD);
+      }
+      if (free_file)
+      {
+        cursor->ha_external_lock(current_session, F_UNLCK);
+        cursor->close();
+        delete cursor;
+      }
+    }
+    delete_dynamic(&ranges); /* ranges are allocated in alloc */
+    free_root(&alloc,MYF(0));
+  }
+  head->column_bitmaps_set(save_read_set, save_write_set);
+  assert(mrr_buf_desc == NULL);
+  if (mrr_buf_desc)
+  {
+    free(mrr_buf_desc);
+  }
+}
+
+
+int optimizer::QuickRangeSelect::init_ror_merged_scan(bool reuse_handler)
+{
+  Cursor *save_file= cursor, *org_file;
+  Session *session;
+
+  in_ror_merged_scan= 1;
+  if (reuse_handler)
+  {
+    if (init() || reset())
+    {
+      return 0;
+    }
+    head->column_bitmaps_set(&column_bitmap, &column_bitmap);
+    goto end;
+  }
+
+  /* Create a separate Cursor object for this quick select */
+  if (free_file)
+  {
+    /* already have own 'Cursor' object. */
+    return 0;
+  }
+
+  session= head->in_use;
+  if (! (cursor= head->cursor->clone(session->mem_root)))
+  {
+    /*
+      Manually set the error flag. Note: there seems to be quite a few
+      places where a failure could cause the server to "hang" the client by
+      sending no response to a query. ATM those are not real errors because
+      the storage engine calls in question happen to never fail with the
+      existing storage engines.
+    */
+    my_error(ER_OUT_OF_RESOURCES, MYF(0));
+    /* Caller will free the memory */
+    goto failure;
+  }
+
+  head->column_bitmaps_set(&column_bitmap, &column_bitmap);
+
+  if (cursor->ha_external_lock(session, F_RDLCK))
+    goto failure;
+
+  if (init() || reset())
+  {
+    cursor->ha_external_lock(session, F_UNLCK);
+    cursor->close();
+    goto failure;
+  }
+  free_file= true;
+  last_rowid= cursor->ref;
+
+end:
+  /*
+    We are only going to read key fields and call position() on 'cursor'
+    The following sets head->tmp_set to only use this key and then updates
+    head->read_set and head->write_set to use this bitmap.
+    The now bitmap is stored in 'column_bitmap' which is used in ::get_next()
+  */
+  org_file= head->cursor;
+  head->cursor= cursor;
+  /* We don't have to set 'head->keyread' here as the 'cursor' is unique */
+  if (! head->no_keyread)
+  {
+    head->key_read= 1;
+    head->mark_columns_used_by_index(index);
+  }
+  head->prepare_for_position();
+  head->cursor= org_file;
+  column_bitmap= *head->read_set;
+  head->column_bitmaps_set(&column_bitmap, &column_bitmap);
+
+  return 0;
+
+failure:
+  head->column_bitmaps_set(save_read_set, save_write_set);
+  delete cursor;
+  cursor= save_file;
+  return 0;
+}
+
+
+void optimizer::QuickRangeSelect::save_last_pos()
+{
+  cursor->position(record);
+}
+
+
+bool optimizer::QuickRangeSelect::unique_key_range()
+{
+  if (ranges.elements == 1)
+  {
+    optimizer::QuickRange *tmp= *((optimizer::QuickRange**)ranges.buffer);
+    if ((tmp->flag & (EQ_RANGE | NULL_RANGE)) == EQ_RANGE)
+    {
+      KEY *key=head->key_info+index;
+      return ((key->flags & (HA_NOSAME)) == HA_NOSAME &&
+	      key->key_length == tmp->min_length);
+    }
+  }
+  return false;
+}
+
+
+int optimizer::QuickRangeSelect::reset()
+{
+  uint32_t buf_size= 0;
+  unsigned char *mrange_buff= NULL;
+  int error= 0;
+  HANDLER_BUFFER empty_buf;
+  last_range= NULL;
+  cur_range= (optimizer::QuickRange**) ranges.buffer;
+
+  if (cursor->inited == Cursor::NONE && (error= cursor->ha_index_init(index, 1)))
+  {
+    return error;
+  }
+
+  /* Allocate buffer if we need one but haven't allocated it yet */
+  if (mrr_buf_size && ! mrr_buf_desc)
+  {
+    buf_size= mrr_buf_size;
+    while (buf_size && ! memory::multi_malloc(false,
+                                              &mrr_buf_desc, 
+                                              sizeof(*mrr_buf_desc),
+                                              &mrange_buff, 
+                                              buf_size,
+                                              NULL))
+    {
+      /* Try to shrink the buffers until both are 0. */
+      buf_size/= 2;
+    }
+    if (! mrr_buf_desc)
+    {
+      return HA_ERR_OUT_OF_MEM;
+    }
+
+    /* Initialize the Cursor buffer. */
+    mrr_buf_desc->buffer= mrange_buff;
+    mrr_buf_desc->buffer_end= mrange_buff + buf_size;
+    mrr_buf_desc->end_of_used_area= mrange_buff;
+  }
+
+  if (! mrr_buf_desc)
+  {
+    empty_buf.buffer= NULL;
+    empty_buf.buffer_end= NULL;
+    empty_buf.end_of_used_area= NULL;
+  }
+
+  if (sorted)
+  {
+     mrr_flags|= HA_MRR_SORTED;
+  }
+  RANGE_SEQ_IF seq_funcs= {
+    optimizer::quick_range_seq_init, 
+    optimizer::quick_range_seq_next
+  };
+  error= cursor->multi_range_read_init(&seq_funcs, 
+                                       (void*) this, 
+                                       ranges.elements,
+                                       mrr_flags, 
+                                       mrr_buf_desc ? mrr_buf_desc : &empty_buf);
+  return error;
+}
+
+
+int optimizer::QuickRangeSelect::get_next()
+{
+  char *dummy= NULL;
+  if (in_ror_merged_scan)
+  {
+    /*
+      We don't need to signal the bitmap change as the bitmap is always the
+      same for this head->cursor
+    */
+    head->column_bitmaps_set(&column_bitmap, &column_bitmap);
+  }
+
+  int result= cursor->multi_range_read_next(&dummy);
+
+  if (in_ror_merged_scan)
+  {
+    /* Restore bitmaps set on entry */
+    head->column_bitmaps_set(save_read_set, save_write_set);
+  }
+  return result;
+}
+
+
+int optimizer::QuickRangeSelect::get_next_prefix(uint32_t prefix_length,
+                                                 key_part_map keypart_map,
+                                                 unsigned char *cur_prefix)
+{
+  for (;;)
+  {
+    int result;
+    key_range start_key, end_key;
+    if (last_range)
+    {
+      /* Read the next record in the same range with prefix after cur_prefix. */
+      assert(cur_prefix != 0);
+      result= cursor->index_read_map(record, 
+                                     cur_prefix, 
+                                     keypart_map,
+                                     HA_READ_AFTER_KEY);
+      if (result || (cursor->compare_key(cursor->end_range) <= 0))
+        return result;
+    }
+
+    uint32_t count= ranges.elements - (cur_range - (optimizer::QuickRange**) ranges.buffer);
+    if (count == 0)
+    {
+      /* Ranges have already been used up before. None is left for read. */
+      last_range= 0;
+      return HA_ERR_END_OF_FILE;
+    }
+    last_range= *(cur_range++);
+
+    start_key.key= (const unsigned char*) last_range->min_key;
+    start_key.length= min(last_range->min_length, (uint16_t)prefix_length);
+    start_key.keypart_map= last_range->min_keypart_map & keypart_map;
+    start_key.flag= ((last_range->flag & NEAR_MIN) ? HA_READ_AFTER_KEY :
+		                                                (last_range->flag & EQ_RANGE) ?
+		                                                HA_READ_KEY_EXACT : HA_READ_KEY_OR_NEXT);
+    end_key.key= (const unsigned char*) last_range->max_key;
+    end_key.length= min(last_range->max_length, (uint16_t)prefix_length);
+    end_key.keypart_map= last_range->max_keypart_map & keypart_map;
+    /*
+      We use READ_AFTER_KEY here because if we are reading on a key
+      prefix we want to find all keys with this prefix
+    */
+    end_key.flag= (last_range->flag & NEAR_MAX ? HA_READ_BEFORE_KEY :
+		                                             HA_READ_AFTER_KEY);
+
+    result= cursor->read_range_first(last_range->min_keypart_map ? &start_key : 0,
+				                             last_range->max_keypart_map ? &end_key : 0,
+                                     test(last_range->flag & EQ_RANGE),
+				                             sorted);
+    if (last_range->flag == (UNIQUE_RANGE | EQ_RANGE))
+      last_range= 0; // Stop searching
+
+    if (result != HA_ERR_END_OF_FILE)
+      return result;
+    last_range= 0; // No matching rows; go to next range
+  }
+}
+
+
+bool optimizer::QuickRangeSelect::row_in_ranges()
+{
+  optimizer::QuickRange *res= NULL;
+  uint32_t min= 0;
+  uint32_t max= ranges.elements - 1;
+  uint32_t mid= (max + min) / 2;
+
+  while (min != max)
+  {
+    if (cmp_next(*(optimizer::QuickRange**)dynamic_array_ptr(&ranges, mid)))
+    {
+      /* current row value > mid->max */
+      min= mid + 1;
+    }
+    else
+      max= mid;
+    mid= (min + max) / 2;
+  }
+  res= *(optimizer::QuickRange**)dynamic_array_ptr(&ranges, mid);
+  return (! cmp_next(res) && ! cmp_prev(res));
+}
+
+
+int optimizer::QuickRangeSelect::cmp_next(optimizer::QuickRange *range_arg)
+{
+  if (range_arg->flag & NO_MAX_RANGE)
+    return 0;                                   /* key can't be to large */
+
+  KEY_PART *key_part= key_parts;
+  uint32_t store_length;
+
+  for (unsigned char *key=range_arg->max_key, *end=key+range_arg->max_length;
+       key < end;
+       key+= store_length, key_part++)
+  {
+    int cmp;
+    store_length= key_part->store_length;
+    if (key_part->null_bit)
+    {
+      if (*key)
+      {
+        if (! key_part->field->is_null())
+          return 1;
+        continue;
+      }
+      else if (key_part->field->is_null())
+        return 0;
+      key++;					// Skip null byte
+      store_length--;
+    }
+    if ((cmp= key_part->field->key_cmp(key, key_part->length)) < 0)
+      return 0;
+    if (cmp > 0)
+      return 1;
+  }
+  return (range_arg->flag & NEAR_MAX) ? 1 : 0;          // Exact match
+}
+
+
+int optimizer::QuickRangeSelect::cmp_prev(optimizer::QuickRange *range_arg)
+{
+  int cmp;
+  if (range_arg->flag & NO_MIN_RANGE)
+    return 0;					/* key can't be to small */
+
+  cmp= key_cmp(key_part_info, 
+               range_arg->min_key,
+               range_arg->min_length);
+  if (cmp > 0 || (cmp == 0 && (range_arg->flag & NEAR_MIN) == false))
+    return 0;
+  return 1;                                     // outside of range
+}
+
+
+void optimizer::QuickRangeSelect::add_info_string(String *str)
+{
+  KEY *key_info= head->key_info + index;
+  str->append(key_info->name);
+}
+
+
+void optimizer::QuickRangeSelect::add_keys_and_lengths(String *key_names,
+                                                       String *used_lengths)
+{
+  char buf[64];
+  uint32_t length;
+  KEY *key_info= head->key_info + index;
+  key_names->append(key_info->name);
+  length= int64_t2str(max_used_key_length, buf, 10) - buf;
+  used_lengths->append(buf, length);
+}
+
+
+/*
+  This is a hack: we inherit from QUICK_SELECT so that we can use the
+  get_next() interface, but we have to hold a pointer to the original
+  QUICK_SELECT because its data are used all over the place.  What
+  should be done is to factor out the data that is needed into a base
+  class (QUICK_SELECT), and then have two subclasses (_ASC and _DESC)
+  which handle the ranges and implement the get_next() function.  But
+  for now, this seems to work right at least.
+ */
+optimizer::QuickSelectDescending::QuickSelectDescending(optimizer::QuickRangeSelect *q, uint32_t, bool *)
+  :
+    optimizer::QuickRangeSelect(*q), 
+    rev_it(rev_ranges)
+{
+  optimizer::QuickRange *r= NULL;
+
+  optimizer::QuickRange **pr= (optimizer::QuickRange**)ranges.buffer;
+  optimizer::QuickRange **end_range= pr + ranges.elements;
+  for (; pr != end_range; pr++)
+    rev_ranges.push_front(*pr);
+
+  /* Remove EQ_RANGE flag for keys that are not using the full key */
+  for (r = rev_it++; r; r= rev_it++)
+  {
+    if ((r->flag & EQ_RANGE) &&
+        head->key_info[index].key_length != r->max_length)
+      r->flag&= ~EQ_RANGE;
+  }
+  rev_it.rewind();
+  q->dont_free= 1;				// Don't free shared mem
+  delete q;
+}
+
+
+int optimizer::QuickSelectDescending::get_next()
+{
+  /* The max key is handled as follows:
+   *   - if there is NO_MAX_RANGE, start at the end and move backwards
+   *   - if it is an EQ_RANGE, which means that max key covers the entire
+   *     key, go directly to the key and read through it (sorting backwards is
+   *     same as sorting forwards)
+   *   - if it is NEAR_MAX, go to the key or next, step back once, and
+   *     move backwards
+   *   - otherwise (not NEAR_MAX == include the key), go after the key,
+   *     step back once, and move backwards
+   */
+  for (;;)
+  {
+    int result;
+    if (last_range)
+    {						// Already read through key
+      result= ((last_range->flag & EQ_RANGE) ?
+		           cursor->index_next_same(record, last_range->min_key,
+					                             last_range->min_length) :
+		           cursor->index_prev(record));
+      if (! result)
+      {
+        if (cmp_prev(*rev_it.ref()) == 0)
+          return 0;
+      }
+      else if (result != HA_ERR_END_OF_FILE)
+        return result;
+    }
+
+    if (! (last_range= rev_it++))
+      return HA_ERR_END_OF_FILE;		// All ranges used
+
+    if (last_range->flag & NO_MAX_RANGE)        // Read last record
+    {
+      int local_error;
+      if ((local_error= cursor->index_last(record)))
+        return local_error;	// Empty table
+      if (cmp_prev(last_range) == 0)
+        return 0;
+      last_range= 0; // No match; go to next range
+      continue;
+    }
+
+    if (last_range->flag & EQ_RANGE)
+    {
+      result = cursor->index_read_map(record, 
+                                      last_range->max_key,
+                                      last_range->max_keypart_map,
+                                      HA_READ_KEY_EXACT);
+    }
+    else
+    {
+      assert(last_range->flag & NEAR_MAX ||
+             range_reads_after_key(last_range));
+      result= cursor->index_read_map(record, 
+                                     last_range->max_key,
+                                     last_range->max_keypart_map,
+                                     ((last_range->flag & NEAR_MAX) ?
+                                      HA_READ_BEFORE_KEY :
+                                      HA_READ_PREFIX_LAST_OR_PREV));
+    }
+    if (result)
+    {
+      if (result != HA_ERR_KEY_NOT_FOUND && result != HA_ERR_END_OF_FILE)
+        return result;
+      last_range= 0;                            // Not found, to next range
+      continue;
+    }
+    if (cmp_prev(last_range) == 0)
+    {
+      if (last_range->flag == (UNIQUE_RANGE | EQ_RANGE))
+        last_range= 0;				// Stop searching
+      return 0;				// Found key is in range
+    }
+    last_range= 0;                              // To next range
+  }
+}
+
+
+/*
+ * true if this range will require using HA_READ_AFTER_KEY
+   See comment in get_next() about this
+ */
+bool optimizer::QuickSelectDescending::range_reads_after_key(optimizer::QuickRange *range_arg)
+{
+  return ((range_arg->flag & (NO_MAX_RANGE | NEAR_MAX)) ||
+	        ! (range_arg->flag & EQ_RANGE) ||
+	        head->key_info[index].key_length != range_arg->max_length) ? 1 : 0;
+}
+
+

=== added file 'drizzled/optimizer/quick_range_select.h'
--- drizzled/optimizer/quick_range_select.h	1970-01-01 00:00:00 +0000
+++ drizzled/optimizer/quick_range_select.h	2009-12-07 21:50:09 +0000
@@ -0,0 +1,311 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008-2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_OPTIMIZER_QUICK_RANGE_SELECT_H
+#define DRIZZLED_OPTIMIZER_QUICK_RANGE_SELECT_H
+
+#include "drizzled/optimizer/range.h"
+
+class Cursor;
+
+namespace drizzled
+{
+
+namespace optimizer
+{
+
+/**
+ * Quick select that does a range scan on a single key. 
+ *
+ * The records are returned in key order.
+ * 
+ */
+class QuickRangeSelect : public QuickSelectInterface
+{
+protected:
+  Cursor *cursor;
+  DYNAMIC_ARRAY ranges; /**< ordered array of range ptrs */
+
+  /** Members to deal with case when this quick select is a ROR-merged scan */
+  bool in_ror_merged_scan;
+  MyBitmap column_bitmap;
+  MyBitmap *save_read_set;
+  MyBitmap *save_write_set;
+  bool free_file; /**< True when this->file is "owned" by this quick select */
+
+  /* Range pointers to be used when not using MRR interface */
+  QuickRange **cur_range; /**< current element in ranges  */
+  QuickRange *last_range;
+
+  /** Members needed to use the MRR interface */
+  QuickRangeSequenceContext qr_traversal_ctx;
+  uint32_t mrr_buf_size; /**< copy from session->variables.read_rnd_buff_size */
+  HANDLER_BUFFER *mrr_buf_desc; /**< the Cursor buffer */
+
+  /** Info about index we're scanning */
+  KEY_PART *key_parts;
+  KEY_PART_INFO *key_part_info;
+
+  bool dont_free; /**< Used by QuickSelectDescending */
+
+  /**
+   * Compare if found key is over max-value
+   * @return 0 if key <= range->max_key
+   * @todo: Figure out why can't this function be as simple as cmp_prev().
+   */
+  int cmp_next(QuickRange *range);
+
+  /**
+   * @return 0 if found key is inside range (found key >= range->min_key).
+   */
+  int cmp_prev(QuickRange *range);
+
+  /**
+   * Check if current row will be retrieved by this QuickRangeSelect
+   *
+   * NOTES
+   * It is assumed that currently a scan is being done on another index
+   * which reads all necessary parts of the index that is scanned by this
+   * quick select.
+   * The implementation does a binary search on sorted array of disjoint
+   * ranges, without taking size of range into account.
+   *
+   * This function is used to filter out clustered PK scan rows in
+   * index_merge quick select.
+   *
+   * RETURN
+   * @retval true  if current row will be retrieved by this quick select
+   * false if not
+   */
+  bool row_in_ranges();
+
+public:
+
+  uint32_t mrr_flags; /**< Flags to be used with MRR interface */
+
+  MEM_ROOT alloc;
+
+  QuickRangeSelect(Session *session,
+                     Table *table,
+                     uint32_t index_arg,
+                     bool no_alloc,
+                     MEM_ROOT *parent_alloc,
+                     bool *create_err);
+
+  ~QuickRangeSelect();
+
+  int init();
+
+  int reset(void);
+
+  /**
+   * Get next possible record using quick-struct.
+   *
+   * SYNOPSIS
+   * QuickRangeSelect::get_next()
+   *
+   * NOTES
+   * Record is read into table->record[0]
+   *
+   * RETURN
+   * @retval 0			Found row
+   * @retval HA_ERR_END_OF_FILE	No (more) rows in range
+   * @retaval #	Error code
+   */
+  int get_next();
+
+  void range_end();
+
+  /**
+   * Get the next record with a different prefix.
+   *
+   * SYNOPSIS
+   * QuickRangeSelect::get_next_prefix()
+   * @param[in] prefix_length  length of cur_prefix
+   * @param[in] cur_prefix     prefix of a key to be searched for
+   *
+   * DESCRIPTION
+   * Each subsequent call to the method retrieves the first record that has a
+   * prefix with length prefix_length different from cur_prefix, such that the
+   * record with the new prefix is within the ranges described by
+   * this->ranges. The record found is stored into the buffer pointed by
+   * this->record.
+   * The method is useful for GROUP-BY queries with range conditions to
+   * discover the prefix of the next group that satisfies the range conditions.
+   *
+   * @todo
+   * This method is a modified copy of QuickRangeSelect::get_next(), so both
+   * methods should be unified into a more general one to reduce code
+   * duplication.
+   *
+   * RETURN
+   * @retval 0                  on success
+   * @retval HA_ERR_END_OF_FILE if returned all keys
+   * @retval other              if some error occurred
+   */
+  int get_next_prefix(uint32_t prefix_length,
+                      key_part_map keypart_map,
+                      unsigned char *cur_prefix);
+
+  bool reverse_sorted()
+  {
+    return false;
+  }
+
+  /**
+   * @return true if there is only one range and this uses the whole primary key
+   */
+  bool unique_key_range();
+
+  /**
+   * Initialize this quick select to be a ROR-merged scan.
+   *
+   * SYNOPSIS
+   * QuickRangeSelect::init_ror_merged_scan()
+   * @param[in] reuse_handler If true, use head->cursor, otherwise create a separate Cursor object
+   *
+   * NOTES
+   * This function creates and prepares for subsequent use a separate Cursor
+   * object if it can't reuse head->cursor. The reason for this is that during
+   * ROR-merge several key scans are performed simultaneously, and a single
+   * Cursor is only capable of preserving context of a single key scan.
+   *
+   * In ROR-merge the quick select doing merge does full records retrieval,
+   * merged quick selects read only keys.
+   *
+   * RETURN
+   * @reval 0  ROR child scan initialized, ok to use.
+   * @retval 1  error
+   */
+  int init_ror_merged_scan(bool reuse_handler);
+
+  void save_last_pos();
+
+  int get_type()
+  {
+    return QS_TYPE_RANGE;
+  }
+
+  void add_keys_and_lengths(String *key_names, String *used_lengths);
+
+  void add_info_string(String *str);
+
+  void resetCursor()
+  {
+    cursor= NULL;
+  }
+
+private:
+
+  /* Used only by QuickSelectDescending */
+  QuickRangeSelect(const QuickRangeSelect& org) : QuickSelectInterface()
+  {
+    memmove(this, &org, sizeof(*this));
+    /*
+      Use default MRR implementation for reverse scans. No table engine
+      currently can do an MRR scan with output in reverse index order.
+    */
+    mrr_buf_desc= NULL;
+    mrr_flags|= HA_MRR_USE_DEFAULT_IMPL;
+    mrr_buf_size= 0;
+  }
+
+  friend class ::TRP_ROR_INTERSECT; 
+
+  friend
+  QuickRangeSelect *get_quick_select_for_ref(Session *session, Table *table,
+                                             struct table_reference_st *ref,
+                                             ha_rows records);
+
+  friend bool get_quick_keys(PARAM *param, 
+                             QuickRangeSelect *quick,
+                             KEY_PART *key, 
+                             SEL_ARG *key_tree,
+                             unsigned char *min_key, 
+                             uint32_t min_key_flag,
+                             unsigned char *max_key, 
+                             uint32_t max_key_flag);
+
+  friend QuickRangeSelect *get_quick_select(PARAM*,uint32_t idx,
+                                            SEL_ARG *key_tree,
+                                            uint32_t mrr_flags,
+                                            uint32_t mrr_buf_size,
+                                            MEM_ROOT *alloc);
+  friend class QuickSelectDescending;
+
+  friend class QuickIndexMergeSelect;
+
+  friend class QUICK_ROR_INTERSECT_SELECT;
+
+  friend class QUICK_GROUP_MIN_MAX_SELECT;
+
+  friend uint32_t quick_range_seq_next(range_seq_t rseq, KEY_MULTI_RANGE *range);
+
+  friend range_seq_t quick_range_seq_init(void *init_param,
+                                          uint32_t n_ranges, 
+                                          uint32_t flags);
+
+  friend void select_describe(JOIN *join, 
+                              bool need_tmp_table, 
+                              bool need_order,
+                              bool distinct,
+                              const char *message);
+};
+
+class QuickSelectDescending : public QuickRangeSelect
+{
+public:
+
+  QuickSelectDescending(QuickRangeSelect *q, 
+                        uint32_t used_key_parts,
+                        bool *create_err);
+
+  int get_next();
+
+  bool reverse_sorted() 
+  { 
+    return true; 
+  }
+
+  int get_type() 
+  { 
+    return QS_TYPE_RANGE_DESC;
+  }
+
+private:
+
+  bool range_reads_after_key(QuickRange *range);
+
+  int reset(void) 
+  { 
+    rev_it.rewind(); 
+    return QuickRangeSelect::reset();
+  }
+
+  List<QuickRange> rev_ranges;
+
+  List_iterator<QuickRange> rev_it;
+
+};
+
+} /* namespace optimizer */
+
+} /* namespace drizzled */
+
+#endif /* DRIZZLED_OPTIMIZER_QUICK_RANGE_SELECT_H */

=== modified file 'drizzled/optimizer/range.cc'
--- drizzled/optimizer/range.cc	2009-12-07 19:18:46 +0000
+++ drizzled/optimizer/range.cc	2009-12-07 21:50:09 +0000
@@ -117,6 +117,9 @@
 #include "drizzled/check_stack_overrun.h"
 #include "drizzled/optimizer/sum.h"
 #include "drizzled/optimizer/range.h"
+#include "drizzled/optimizer/quick_range.h"
+#include "drizzled/optimizer/quick_range_select.h"
+#include "drizzled/optimizer/quick_index_merge_select.h"
 #include "drizzled/records.h"
 
 #include "drizzled/temporal.h" /* Needed in get_mm_leaf() for timestamp -> datetime comparisons */
@@ -141,12 +144,6 @@
     return static_cast<ha_rows>(x);
 }
 
-extern "C" int refpos_order_cmp(void *arg, const void *a, const void *b)
-{
-  Cursor *cursor= (Cursor*)arg;
-  return cursor->cmp_ref((const unsigned char *) a, (const unsigned char *) b);
-}
-
 static int sel_cmp(Field *f, unsigned char *a, unsigned char *b, uint8_t a_flag, uint8_t b_flag);
 
 static unsigned char is_null_string[2]= {1,0};
@@ -1288,165 +1285,6 @@
     used_key_parts(0)
 {}
 
-optimizer::QuickRangeSelect::QuickRangeSelect(Session *session, 
-                                                  Table *table, 
-                                                  uint32_t key_nr,
-                                                  bool no_alloc, 
-                                                  MEM_ROOT *parent_alloc,
-                                                  bool *create_error)
-  :
-    free_file(0),
-    cur_range(NULL),
-    last_range(0),
-    dont_free(0)
-{
-  my_bitmap_map *bitmap= NULL;
-
-  in_ror_merged_scan= 0;
-  sorted= 0;
-  index= key_nr;
-  head= table;
-  key_part_info= head->key_info[index].key_part;
-  my_init_dynamic_array(&ranges, sizeof(optimizer::QuickRange*), 16, 16);
-
-  /* 'session' is not accessible in QuickRangeSelect::reset(). */
-  mrr_buf_size= session->variables.read_rnd_buff_size;
-  mrr_buf_desc= NULL;
-
-  if (!no_alloc && !parent_alloc)
-  {
-    // Allocates everything through the internal memroot
-    init_sql_alloc(&alloc, session->variables.range_alloc_block_size, 0);
-    session->mem_root= &alloc;
-  }
-  else
-    memset(&alloc, 0, sizeof(alloc));
-  cursor= head->cursor;
-  record= head->record[0];
-  save_read_set= head->read_set;
-  save_write_set= head->write_set;
-
-  /* Allocate a bitmap for used columns. Using sql_alloc instead of malloc
-     simply as a "fix" to the MySQL 6.0 code that also free()s it at the
-     same time we destroy the mem_root.
-   */
-
-  bitmap= reinterpret_cast<my_bitmap_map*>(sql_alloc(head->s->column_bitmap_size));
-  if (! bitmap)
-  {
-    column_bitmap.setBitmap(NULL);
-    *create_error= 1;
-  }
-  else
-  {
-    column_bitmap.init(bitmap, head->s->fields);
-  }
-}
-
-
-int optimizer::QuickRangeSelect::init()
-{
-  if (cursor->inited != Cursor::NONE)
-    cursor->ha_index_or_rnd_end();
-  return (cursor->ha_index_init(index, 1));
-}
-
-
-void optimizer::QuickRangeSelect::range_end()
-{
-  if (cursor->inited != Cursor::NONE)
-    cursor->ha_index_or_rnd_end();
-}
-
-
-optimizer::QuickRangeSelect::~QuickRangeSelect()
-{
-  if (!dont_free)
-  {
-    /* cursor is NULL for CPK scan on covering ROR-intersection */
-    if (cursor)
-    {
-      range_end();
-      if (head->key_read)
-      {
-        head->key_read= 0;
-        cursor->extra(HA_EXTRA_NO_KEYREAD);
-      }
-      if (free_file)
-      {
-        cursor->ha_external_lock(current_session, F_UNLCK);
-        cursor->close();
-        delete cursor;
-      }
-    }
-    delete_dynamic(&ranges); /* ranges are allocated in alloc */
-    free_root(&alloc,MYF(0));
-  }
-  head->column_bitmaps_set(save_read_set, save_write_set);
-  assert(mrr_buf_desc == NULL);
-  if (mrr_buf_desc)
-  {
-    free(mrr_buf_desc);
-  }
-}
-
-
-optimizer::QUICK_INDEX_MERGE_SELECT::QUICK_INDEX_MERGE_SELECT(Session *session_param,
-                                                              Table *table)
-  :
-    pk_quick_select(NULL), 
-    session(session_param)
-{
-  index= MAX_KEY;
-  head= table;
-  memset(&read_record, 0, sizeof(read_record));
-  init_sql_alloc(&alloc, session->variables.range_alloc_block_size, 0);
-  return;
-}
-
-int optimizer::QUICK_INDEX_MERGE_SELECT::init()
-{
-  return 0;
-}
-
-int optimizer::QUICK_INDEX_MERGE_SELECT::reset()
-{
-  return (read_keys_and_merge());
-}
-
-bool
-optimizer::QUICK_INDEX_MERGE_SELECT::push_quick_back(optimizer::QuickRangeSelect *quick_sel_range)
-{
-  /*
-    Save quick_select that does scan on clustered primary key as it will be
-    processed separately.
-  */
-  if (head->cursor->primary_key_is_clustered() &&
-      quick_sel_range->index == head->s->primary_key)
-  {
-    pk_quick_select= quick_sel_range;
-  }
-  else
-  {
-    return quick_selects.push_back(quick_sel_range);
-  }
-  return 0;
-}
-
-optimizer::QUICK_INDEX_MERGE_SELECT::~QUICK_INDEX_MERGE_SELECT()
-{
-  List_iterator_fast<optimizer::QuickRangeSelect> quick_it(quick_selects);
-  optimizer::QuickRangeSelect* quick;
-  quick_it.rewind();
-  while ((quick= quick_it++))
-  {
-    quick->cursor= NULL;
-  }
-  quick_selects.delete_elements();
-  delete pk_quick_select;
-  free_root(&alloc,MYF(0));
-  return;
-}
 
 
 optimizer::QUICK_ROR_INTERSECT_SELECT::QUICK_ROR_INTERSECT_SELECT(Session *session_param,
@@ -1493,116 +1331,6 @@
 
 
 /*
-  Initialize this quick select to be a ROR-merged scan.
-
-  SYNOPSIS
-    QuickRangeSelect::init_ror_merged_scan()
-      reuse_handler If true, use head->cursor, otherwise create a separate
-                    Cursor object
-
-  NOTES
-    This function creates and prepares for subsequent use a separate Cursor
-    object if it can't reuse head->cursor. The reason for this is that during
-    ROR-merge several key scans are performed simultaneously, and a single
-    Cursor is only capable of preserving context of a single key scan.
-
-    In ROR-merge the quick select doing merge does full records retrieval,
-    merged quick selects read only keys.
-
-  RETURN
-    0  ROR child scan initialized, ok to use.
-    1  error
-*/
-
-int optimizer::QuickRangeSelect::init_ror_merged_scan(bool reuse_handler)
-{
-  Cursor *save_file= cursor, *org_file;
-  Session *session;
-
-  in_ror_merged_scan= 1;
-  if (reuse_handler)
-  {
-    if (init() || reset())
-    {
-      return 0;
-    }
-    head->column_bitmaps_set(&column_bitmap, &column_bitmap);
-    goto end;
-  }
-
-  /* Create a separate Cursor object for this quick select */
-  if (free_file)
-  {
-    /* already have own 'Cursor' object. */
-    return 0;
-  }
-
-  session= head->in_use;
-  if (! (cursor= head->cursor->clone(session->mem_root)))
-  {
-    /*
-      Manually set the error flag. Note: there seems to be quite a few
-      places where a failure could cause the server to "hang" the client by
-      sending no response to a query. ATM those are not real errors because
-      the storage engine calls in question happen to never fail with the
-      existing storage engines.
-    */
-    my_error(ER_OUT_OF_RESOURCES, MYF(0));
-    /* Caller will free the memory */
-    goto failure;
-  }
-
-  head->column_bitmaps_set(&column_bitmap, &column_bitmap);
-
-  if (cursor->ha_external_lock(session, F_RDLCK))
-    goto failure;
-
-  if (init() || reset())
-  {
-    cursor->ha_external_lock(session, F_UNLCK);
-    cursor->close();
-    goto failure;
-  }
-  free_file= true;
-  last_rowid= cursor->ref;
-
-end:
-  /*
-    We are only going to read key fields and call position() on 'cursor'
-    The following sets head->tmp_set to only use this key and then updates
-    head->read_set and head->write_set to use this bitmap.
-    The now bitmap is stored in 'column_bitmap' which is used in ::get_next()
-  */
-  org_file= head->cursor;
-  head->cursor= cursor;
-  /* We don't have to set 'head->keyread' here as the 'cursor' is unique */
-  if (! head->no_keyread)
-  {
-    head->key_read= 1;
-    head->mark_columns_used_by_index(index);
-  }
-  head->prepare_for_position();
-  head->cursor= org_file;
-  column_bitmap= *head->read_set;
-  head->column_bitmaps_set(&column_bitmap, &column_bitmap);
-
-  return 0;
-
-failure:
-  head->column_bitmaps_set(save_read_set, save_write_set);
-  delete cursor;
-  cursor= save_file;
-  return 0;
-}
-
-
-void optimizer::QuickRangeSelect::save_last_pos()
-{
-  cursor->position(record);
-}
-
-
-/*
   Initialize this quick select to be a part of a ROR-merged scan.
   SYNOPSIS
     QUICK_ROR_INTERSECT_SELECT::init_ror_merged_scan()
@@ -1615,7 +1343,7 @@
 int optimizer::QUICK_ROR_INTERSECT_SELECT::init_ror_merged_scan(bool reuse_handler)
 {
   List_iterator_fast<optimizer::QuickRangeSelect> quick_it(quick_selects);
-  optimizer::QuickRangeSelect* quick;
+  optimizer::QuickRangeSelect *quick= NULL;
 
   /* Initialize all merged "children" quick selects */
   assert(!need_to_fetch_row || reuse_handler);
@@ -1666,7 +1394,7 @@
   }
   scans_inited= true;
   List_iterator_fast<optimizer::QuickRangeSelect> it(quick_selects);
-  optimizer::QuickRangeSelect *quick;
+  optimizer::QuickRangeSelect *quick= NULL;
   while ((quick= it++))
   {
     quick->reset();
@@ -1853,17 +1581,6 @@
 }
 
 
-optimizer::QuickRange::QuickRange()
-  :
-    min_key(0),
-    max_key(0),
-    min_length(0),
-    max_length(0),
-    flag(NO_MIN_RANGE | NO_MAX_RANGE),
-    min_keypart_map(0), 
-    max_keypart_map(0)
-{}
-
 SEL_ARG::SEL_ARG(SEL_ARG &arg) :Sql_alloc()
 {
   type=arg.type;
@@ -2259,7 +1976,7 @@
 
 
 /*
-  Plan for QUICK_INDEX_MERGE_SELECT scan.
+  Plan for QuickIndexMergeSelect scan.
   QUICK_ROR_INTERSECT_SELECT always retrieves full rows, so retrieve_full_rows
   is ignored by make_quick.
 */
@@ -3892,10 +3609,10 @@
 
 optimizer::QuickSelectInterface *TRP_INDEX_MERGE::make_quick(PARAM *param, bool, MEM_ROOT *)
 {
-  optimizer::QUICK_INDEX_MERGE_SELECT *quick_imerge;
-  optimizer::QuickRangeSelect *quick;
+  optimizer::QuickIndexMergeSelect *quick_imerge;
+  optimizer::QuickRangeSelect *quick= NULL;
   /* index_merge always retrieves full rows, ignore retrieve_full_rows */
-  if (! (quick_imerge= new optimizer::QUICK_INDEX_MERGE_SELECT(param->session, param->table)))
+  if (! (quick_imerge= new optimizer::QuickIndexMergeSelect(param->session, param->table)))
   {
     return NULL;
   }
@@ -6934,27 +6651,6 @@
 }
 
 /*
-  Return 1 if there is only one range and this uses the whole primary key
-*/
-
-bool optimizer::QuickRangeSelect::unique_key_range()
-{
-  if (ranges.elements == 1)
-  {
-    optimizer::QuickRange *tmp= *((optimizer::QuickRange**)ranges.buffer);
-    if ((tmp->flag & (EQ_RANGE | NULL_RANGE)) == EQ_RANGE)
-    {
-      KEY *key=head->key_info+index;
-      return ((key->flags & (HA_NOSAME)) == HA_NOSAME &&
-	      key->key_length == tmp->min_length);
-    }
-  }
-  return 0;
-}
-
-
-
-/*
   Return true if any part of the key is NULL
 
   SYNOPSIS
@@ -6986,17 +6682,6 @@
   return is_key_used(head, index, fields);
 }
 
-bool optimizer::QUICK_INDEX_MERGE_SELECT::is_keys_used(const MyBitmap *fields)
-{
-  optimizer::QuickRangeSelect *quick= NULL;
-  List_iterator_fast<QuickRangeSelect> it(quick_selects);
-  while ((quick= it++))
-  {
-    if (is_key_used(head, quick->index, fields))
-      return 1;
-  }
-  return 0;
-}
 
 bool optimizer::QUICK_ROR_INTERSECT_SELECT::is_keys_used(const MyBitmap *fields)
 {
@@ -7143,136 +6828,6 @@
 
 
 /*
-  Perform key scans for all used indexes (except CPK), get rowids and merge
-  them into an ordered non-recurrent sequence of rowids.
-
-  The merge/duplicate removal is performed using Unique class. We put all
-  rowids into Unique, get the sorted sequence and destroy the Unique.
-
-  If table has a clustered primary key that covers all rows (true for bdb
-  and innodb currently) and one of the index_merge scans is a scan on PK,
-  then rows that will be retrieved by PK scan are not put into Unique and
-  primary key scan is not performed here, it is performed later separately.
-
-  RETURN
-    0     OK
-    other error
-*/
-
-int optimizer::QUICK_INDEX_MERGE_SELECT::read_keys_and_merge()
-{
-  List_iterator_fast<optimizer::QuickRangeSelect> cur_quick_it(quick_selects);
-  optimizer::QuickRangeSelect* cur_quick;
-  int result;
-  Unique *unique;
-  Cursor *cursor= head->cursor;
-
-  cursor->extra(HA_EXTRA_KEYREAD);
-  head->prepare_for_position();
-
-  cur_quick_it.rewind();
-  cur_quick= cur_quick_it++;
-  assert(cur_quick != 0);
-
-  /*
-    We reuse the same instance of Cursor so we need to call both init and
-    reset here.
-  */
-  if (cur_quick->init() || cur_quick->reset())
-    return 0;
-
-  unique= new Unique(refpos_order_cmp, (void *)cursor,
-                     cursor->ref_length,
-                     session->variables.sortbuff_size);
-  if (!unique)
-    return 0;
-  for (;;)
-  {
-    while ((result= cur_quick->get_next()) == HA_ERR_END_OF_FILE)
-    {
-      cur_quick->range_end();
-      cur_quick= cur_quick_it++;
-      if (!cur_quick)
-        break;
-
-      if (cur_quick->cursor->inited != Cursor::NONE)
-        cur_quick->cursor->ha_index_end();
-      if (cur_quick->init() || cur_quick->reset())
-        return 0;
-    }
-
-    if (result)
-    {
-      if (result != HA_ERR_END_OF_FILE)
-      {
-        cur_quick->range_end();
-        return result;
-      }
-      break;
-    }
-
-    if (session->killed)
-      return 0;
-
-    /* skip row if it will be retrieved by clustered PK scan */
-    if (pk_quick_select && pk_quick_select->row_in_ranges())
-      continue;
-
-    cur_quick->cursor->position(cur_quick->record);
-    result= unique->unique_add((char*)cur_quick->cursor->ref);
-    if (result)
-      return 0;
-
-  }
-
-  /* ok, all row ids are in Unique */
-  result= unique->get(head);
-  delete unique;
-  doing_pk_scan= false;
-  /* index_merge currently doesn't support "using index" at all */
-  cursor->extra(HA_EXTRA_NO_KEYREAD);
-  /* start table scan */
-  init_read_record(&read_record, session, head, (optimizer::SqlSelect*) 0, 1, 1);
-  return result;
-}
-
-
-/*
-  Get next row for index_merge.
-  NOTES
-    The rows are read from
-      1. rowids stored in Unique.
-      2. QuickRangeSelect with clustered primary key (if any).
-    The sets of rows retrieved in 1) and 2) are guaranteed to be disjoint.
-*/
-
-int optimizer::QUICK_INDEX_MERGE_SELECT::get_next()
-{
-  int result;
-
-  if (doing_pk_scan)
-    return(pk_quick_select->get_next());
-
-  if ((result= read_record.read_record(&read_record)) == -1)
-  {
-    result= HA_ERR_END_OF_FILE;
-    end_read_record(&read_record);
-    /* All rows from Unique have been retrieved, do a clustered PK scan */
-    if (pk_quick_select)
-    {
-      doing_pk_scan= true;
-      if ((result= pk_quick_select->init()) ||
-          (result= pk_quick_select->reset()))
-        return result;
-      return(pk_quick_select->get_next());
-    }
-  }
-
-  return result;
-}
-
-
-/*
   Retrieve next record.
   SYNOPSIS
      QUICK_ROR_INTERSECT_SELECT::get_next()
@@ -7428,69 +6983,6 @@
 }
 
 
-int optimizer::QuickRangeSelect::reset()
-{
-  uint32_t buf_size= 0;
-  unsigned char *mrange_buff= NULL;
-  int error= 0;
-  HANDLER_BUFFER empty_buf;
-  last_range= NULL;
-  cur_range= (optimizer::QuickRange**) ranges.buffer;
-
-  if (cursor->inited == Cursor::NONE && (error= cursor->ha_index_init(index, 1)))
-  {
-    return error;
-  }
-
-  /* Allocate buffer if we need one but haven't allocated it yet */
-  if (mrr_buf_size && ! mrr_buf_desc)
-  {
-    buf_size= mrr_buf_size;
-    while (buf_size && ! memory::multi_malloc(false,
-                                              &mrr_buf_desc, 
-                                              sizeof(*mrr_buf_desc),
-                                              &mrange_buff, 
-                                              buf_size,
-                                              NULL))
-    {
-      /* Try to shrink the buffers until both are 0. */
-      buf_size/= 2;
-    }
-    if (! mrr_buf_desc)
-    {
-      return HA_ERR_OUT_OF_MEM;
-    }
-
-    /* Initialize the Cursor buffer. */
-    mrr_buf_desc->buffer= mrange_buff;
-    mrr_buf_desc->buffer_end= mrange_buff + buf_size;
-    mrr_buf_desc->end_of_used_area= mrange_buff;
-  }
-
-  if (! mrr_buf_desc)
-  {
-    empty_buf.buffer= NULL;
-    empty_buf.buffer_end= NULL;
-    empty_buf.end_of_used_area= NULL;
-  }
-
-  if (sorted)
-  {
-     mrr_flags|= HA_MRR_SORTED;
-  }
-  RANGE_SEQ_IF seq_funcs= {
-    optimizer::quick_range_seq_init, 
-    optimizer::quick_range_seq_next
-  };
-  error= cursor->multi_range_read_init(&seq_funcs, 
-                                       (void*) this, 
-                                       ranges.elements,
-                                       mrr_flags, 
-                                       mrr_buf_desc ? mrr_buf_desc : &empty_buf);
-  return error;
-}
-
-
 /*
   Range sequence interface implementation for array<QuickRange>: initialize
 
@@ -7559,383 +7051,6 @@
 }
 
 
-/*
-  Get next possible record using quick-struct.
-
-  SYNOPSIS
-    QuickRangeSelect::get_next()
-
-  NOTES
-    Record is read into table->record[0]
-
-  RETURN
-    0			Found row
-    HA_ERR_END_OF_FILE	No (more) rows in range
-    #			Error code
-*/
-int optimizer::QuickRangeSelect::get_next()
-{
-  char *dummy= NULL;
-  if (in_ror_merged_scan)
-  {
-    /*
-      We don't need to signal the bitmap change as the bitmap is always the
-      same for this head->cursor
-    */
-    head->column_bitmaps_set(&column_bitmap, &column_bitmap);
-  }
-
-  int result= cursor->multi_range_read_next(&dummy);
-
-  if (in_ror_merged_scan)
-  {
-    /* Restore bitmaps set on entry */
-    head->column_bitmaps_set(save_read_set, save_write_set);
-  }
-  return result;
-}
-
-
-/*
-  Get the next record with a different prefix.
-
-  SYNOPSIS
-    QuickRangeSelect::get_next_prefix()
-    prefix_length  length of cur_prefix
-    cur_prefix     prefix of a key to be searched for
-
-  DESCRIPTION
-    Each subsequent call to the method retrieves the first record that has a
-    prefix with length prefix_length different from cur_prefix, such that the
-    record with the new prefix is within the ranges described by
-    this->ranges. The record found is stored into the buffer pointed by
-    this->record.
-    The method is useful for GROUP-BY queries with range conditions to
-    discover the prefix of the next group that satisfies the range conditions.
-
-  TODO
-    This method is a modified copy of QuickRangeSelect::get_next(), so both
-    methods should be unified into a more general one to reduce code
-    duplication.
-
-  RETURN
-    0                  on success
-    HA_ERR_END_OF_FILE if returned all keys
-    other              if some error occurred
-*/
-int optimizer::QuickRangeSelect::get_next_prefix(uint32_t prefix_length,
-                                                 key_part_map keypart_map,
-                                                 unsigned char *cur_prefix)
-{
-  for (;;)
-  {
-    int result;
-    key_range start_key, end_key;
-    if (last_range)
-    {
-      /* Read the next record in the same range with prefix after cur_prefix. */
-      assert(cur_prefix != 0);
-      result= cursor->index_read_map(record, 
-                                     cur_prefix, 
-                                     keypart_map,
-                                     HA_READ_AFTER_KEY);
-      if (result || (cursor->compare_key(cursor->end_range) <= 0))
-        return result;
-    }
-
-    uint32_t count= ranges.elements - (cur_range - (optimizer::QuickRange**) ranges.buffer);
-    if (count == 0)
-    {
-      /* Ranges have already been used up before. None is left for read. */
-      last_range= 0;
-      return HA_ERR_END_OF_FILE;
-    }
-    last_range= *(cur_range++);
-
-    start_key.key= (const unsigned char*) last_range->min_key;
-    start_key.length= min(last_range->min_length, (uint16_t)prefix_length);
-    start_key.keypart_map= last_range->min_keypart_map & keypart_map;
-    start_key.flag= ((last_range->flag & NEAR_MIN) ? HA_READ_AFTER_KEY :
-		                                                (last_range->flag & EQ_RANGE) ?
-		                                                HA_READ_KEY_EXACT : HA_READ_KEY_OR_NEXT);
-    end_key.key= (const unsigned char*) last_range->max_key;
-    end_key.length= min(last_range->max_length, (uint16_t)prefix_length);
-    end_key.keypart_map= last_range->max_keypart_map & keypart_map;
-    /*
-      We use READ_AFTER_KEY here because if we are reading on a key
-      prefix we want to find all keys with this prefix
-    */
-    end_key.flag= (last_range->flag & NEAR_MAX ? HA_READ_BEFORE_KEY :
-		                                             HA_READ_AFTER_KEY);
-
-    result= cursor->read_range_first(last_range->min_keypart_map ? &start_key : 0,
-				                             last_range->max_keypart_map ? &end_key : 0,
-                                     test(last_range->flag & EQ_RANGE),
-				                             sorted);
-    if (last_range->flag == (UNIQUE_RANGE | EQ_RANGE))
-      last_range= 0; // Stop searching
-
-    if (result != HA_ERR_END_OF_FILE)
-      return result;
-    last_range= 0; // No matching rows; go to next range
-  }
-}
-
-
-/*
-  Check if current row will be retrieved by this QuickRangeSelect
-
-  NOTES
-    It is assumed that currently a scan is being done on another index
-    which reads all necessary parts of the index that is scanned by this
-    quick select.
-    The implementation does a binary search on sorted array of disjoint
-    ranges, without taking size of range into account.
-
-    This function is used to filter out clustered PK scan rows in
-    index_merge quick select.
-
-  RETURN
-    true  if current row will be retrieved by this quick select
-    false if not
-*/
-bool optimizer::QuickRangeSelect::row_in_ranges()
-{
-  optimizer::QuickRange *res= NULL;
-  uint32_t min= 0;
-  uint32_t max= ranges.elements - 1;
-  uint32_t mid= (max + min) / 2;
-
-  while (min != max)
-  {
-    if (cmp_next(*(optimizer::QuickRange**)dynamic_array_ptr(&ranges, mid)))
-    {
-      /* current row value > mid->max */
-      min= mid + 1;
-    }
-    else
-      max= mid;
-    mid= (min + max) / 2;
-  }
-  res= *(optimizer::QuickRange**)dynamic_array_ptr(&ranges, mid);
-  return (! cmp_next(res) && ! cmp_prev(res));
-}
-
-/*
-  This is a hack: we inherit from QUICK_SELECT so that we can use the
-  get_next() interface, but we have to hold a pointer to the original
-  QUICK_SELECT because its data are used all over the place.  What
-  should be done is to factor out the data that is needed into a base
-  class (QUICK_SELECT), and then have two subclasses (_ASC and _DESC)
-  which handle the ranges and implement the get_next() function.  But
-  for now, this seems to work right at least.
- */
-optimizer::QUICK_SELECT_DESC::QUICK_SELECT_DESC(optimizer::QuickRangeSelect *q, uint32_t, bool *)
-  :
-    optimizer::QuickRangeSelect(*q), 
-    rev_it(rev_ranges)
-{
-  optimizer::QuickRange *r= NULL;
-
-  optimizer::QuickRange **pr= (optimizer::QuickRange**)ranges.buffer;
-  optimizer::QuickRange **end_range= pr + ranges.elements;
-  for (; pr != end_range; pr++)
-    rev_ranges.push_front(*pr);
-
-  /* Remove EQ_RANGE flag for keys that are not using the full key */
-  for (r = rev_it++; r; r= rev_it++)
-  {
-    if ((r->flag & EQ_RANGE) &&
-        head->key_info[index].key_length != r->max_length)
-      r->flag&= ~EQ_RANGE;
-  }
-  rev_it.rewind();
-  q->dont_free= 1;				// Don't free shared mem
-  delete q;
-}
-
-
-int optimizer::QUICK_SELECT_DESC::get_next()
-{
-  /* The max key is handled as follows:
-   *   - if there is NO_MAX_RANGE, start at the end and move backwards
-   *   - if it is an EQ_RANGE, which means that max key covers the entire
-   *     key, go directly to the key and read through it (sorting backwards is
-   *     same as sorting forwards)
-   *   - if it is NEAR_MAX, go to the key or next, step back once, and
-   *     move backwards
-   *   - otherwise (not NEAR_MAX == include the key), go after the key,
-   *     step back once, and move backwards
-   */
-  for (;;)
-  {
-    int result;
-    if (last_range)
-    {						// Already read through key
-      result= ((last_range->flag & EQ_RANGE) ?
-		           cursor->index_next_same(record, last_range->min_key,
-					                             last_range->min_length) :
-		           cursor->index_prev(record));
-      if (! result)
-      {
-        if (cmp_prev(*rev_it.ref()) == 0)
-          return 0;
-      }
-      else if (result != HA_ERR_END_OF_FILE)
-        return result;
-    }
-
-    if (! (last_range= rev_it++))
-      return HA_ERR_END_OF_FILE;		// All ranges used
-
-    if (last_range->flag & NO_MAX_RANGE)        // Read last record
-    {
-      int local_error;
-      if ((local_error= cursor->index_last(record)))
-        return local_error;	// Empty table
-      if (cmp_prev(last_range) == 0)
-        return 0;
-      last_range= 0; // No match; go to next range
-      continue;
-    }
-
-    if (last_range->flag & EQ_RANGE)
-    {
-      result = cursor->index_read_map(record, 
-                                      last_range->max_key,
-                                      last_range->max_keypart_map,
-                                      HA_READ_KEY_EXACT);
-    }
-    else
-    {
-      assert(last_range->flag & NEAR_MAX ||
-             range_reads_after_key(last_range));
-      result= cursor->index_read_map(record, 
-                                     last_range->max_key,
-                                     last_range->max_keypart_map,
-                                     ((last_range->flag & NEAR_MAX) ?
-                                      HA_READ_BEFORE_KEY :
-                                      HA_READ_PREFIX_LAST_OR_PREV));
-    }
-    if (result)
-    {
-      if (result != HA_ERR_KEY_NOT_FOUND && result != HA_ERR_END_OF_FILE)
-        return result;
-      last_range= 0;                            // Not found, to next range
-      continue;
-    }
-    if (cmp_prev(last_range) == 0)
-    {
-      if (last_range->flag == (UNIQUE_RANGE | EQ_RANGE))
-        last_range= 0;				// Stop searching
-      return 0;				// Found key is in range
-    }
-    last_range= 0;                              // To next range
-  }
-}
-
-
-/*
-  Compare if found key is over max-value
-  Returns 0 if key <= range->max_key
-  TODO: Figure out why can't this function be as simple as cmp_prev().
-*/
-
-int optimizer::QuickRangeSelect::cmp_next(optimizer::QuickRange *range_arg)
-{
-  if (range_arg->flag & NO_MAX_RANGE)
-    return 0;                                   /* key can't be to large */
-
-  KEY_PART *key_part= key_parts;
-  uint32_t store_length;
-
-  for (unsigned char *key=range_arg->max_key, *end=key+range_arg->max_length;
-       key < end;
-       key+= store_length, key_part++)
-  {
-    int cmp;
-    store_length= key_part->store_length;
-    if (key_part->null_bit)
-    {
-      if (*key)
-      {
-        if (! key_part->field->is_null())
-          return 1;
-        continue;
-      }
-      else if (key_part->field->is_null())
-        return 0;
-      key++;					// Skip null byte
-      store_length--;
-    }
-    if ((cmp= key_part->field->key_cmp(key, key_part->length)) < 0)
-      return 0;
-    if (cmp > 0)
-      return 1;
-  }
-  return (range_arg->flag & NEAR_MAX) ? 1 : 0;          // Exact match
-}
-
-
-/*
-  Returns 0 if found key is inside range (found key >= range->min_key).
-*/
-int optimizer::QuickRangeSelect::cmp_prev(optimizer::QuickRange *range_arg)
-{
-  int cmp;
-  if (range_arg->flag & NO_MIN_RANGE)
-    return 0;					/* key can't be to small */
-
-  cmp= key_cmp(key_part_info, 
-               range_arg->min_key,
-               range_arg->min_length);
-  if (cmp > 0 || (cmp == 0 && (range_arg->flag & NEAR_MIN) == false))
-    return 0;
-  return 1;                                     // outside of range
-}
-
-
-/*
- * true if this range will require using HA_READ_AFTER_KEY
-   See comment in get_next() about this
- */
-bool optimizer::QUICK_SELECT_DESC::range_reads_after_key(optimizer::QuickRange *range_arg)
-{
-  return ((range_arg->flag & (NO_MAX_RANGE | NEAR_MAX)) ||
-	        ! (range_arg->flag & EQ_RANGE) ||
-	        head->key_info[index].key_length != range_arg->max_length) ? 1 : 0;
-}
-
-
-void optimizer::QuickRangeSelect::add_info_string(String *str)
-{
-  KEY *key_info= head->key_info + index;
-  str->append(key_info->name);
-}
-
-void optimizer::QUICK_INDEX_MERGE_SELECT::add_info_string(String *str)
-{
-  optimizer::QuickRangeSelect *quick= NULL;
-  bool first= true;
-  List_iterator_fast<optimizer::QuickRangeSelect> it(quick_selects);
-  str->append(STRING_WITH_LEN("sort_union("));
-  while ((quick= it++))
-  {
-    if (! first)
-      str->append(',');
-    else
-      first= false;
-    quick->add_info_string(str);
-  }
-  if (pk_quick_select)
-  {
-    str->append(',');
-    pk_quick_select->add_info_string(str);
-  }
-  str->append(')');
-}
-
-
 void optimizer::QUICK_ROR_INTERSECT_SELECT::add_info_string(String *str)
 {
   bool first= true;
@@ -7979,54 +7094,6 @@
 }
 
 
-void optimizer::QuickRangeSelect::add_keys_and_lengths(String *key_names,
-                                                       String *used_lengths)
-{
-  char buf[64];
-  uint32_t length;
-  KEY *key_info= head->key_info + index;
-  key_names->append(key_info->name);
-  length= int64_t2str(max_used_key_length, buf, 10) - buf;
-  used_lengths->append(buf, length);
-}
-
-
-void optimizer::QUICK_INDEX_MERGE_SELECT::add_keys_and_lengths(String *key_names,
-                                                               String *used_lengths)
-{
-  char buf[64];
-  uint32_t length;
-  bool first= true;
-  optimizer::QuickRangeSelect *quick= NULL;
-
-  List_iterator_fast<optimizer::QuickRangeSelect> it(quick_selects);
-  while ((quick= it++))
-  {
-    if (first)
-      first= false;
-    else
-    {
-      key_names->append(',');
-      used_lengths->append(',');
-    }
-
-    KEY *key_info= head->key_info + quick->index;
-    key_names->append(key_info->name);
-    length= int64_t2str(quick->max_used_key_length, buf, 10) - buf;
-    used_lengths->append(buf, length);
-  }
-  if (pk_quick_select)
-  {
-    KEY *key_info= head->key_info + pk_quick_select->index;
-    key_names->append(',');
-    key_names->append(key_info->name);
-    length= int64_t2str(pk_quick_select->max_used_key_length, buf, 10) - buf;
-    used_lengths->append(',');
-    used_lengths->append(buf, length);
-  }
-}
-
-
 void optimizer::QUICK_ROR_INTERSECT_SELECT::add_keys_and_lengths(String *key_names,
                                                                  String *used_lengths)
 {
@@ -8776,7 +7843,7 @@
         memset(args, 0, 3 * sizeof(Item*));
         bool inv= false;
         /* Test if this is a comparison of a field and a constant. */
-        if (! optimizer::simple_pred(pred, args, &inv))
+        if (! optimizer::simple_pred(pred, args, inv))
           return false;
 
         /* Check for compatible string comparisons - similar to get_mm_leaf. */

=== modified file 'drizzled/optimizer/range.h'
--- drizzled/optimizer/range.h	2009-12-07 19:18:46 +0000
+++ drizzled/optimizer/range.h	2009-12-07 21:50:09 +0000
@@ -58,36 +58,6 @@
 namespace optimizer
 {
 
-class QuickRange : public Sql_alloc 
-{
-public:
-  unsigned char *min_key;
-  unsigned char *max_key;
-  uint16_t min_length;
-  uint16_t max_length;
-  uint16_t flag;
-  key_part_map min_keypart_map; /**< bitmap of used keyparts in min_key */
-  key_part_map max_keypart_map; /**< bitmap of used keyparts in max_key */
-
-  QuickRange(); /**< Constructor for a "full range" */
-  QuickRange(const unsigned char *min_key_arg,
-              uint32_t min_length_arg,
-              key_part_map min_keypart_map_arg,
-	            const unsigned char *max_key_arg, 
-              uint32_t max_length_arg,
-              key_part_map max_keypart_map_arg,
-	            uint32_t flag_arg)
-    : 
-      min_key((unsigned char*) sql_memdup(min_key_arg,min_length_arg+1)),
-      max_key((unsigned char*) sql_memdup(max_key_arg,max_length_arg+1)),
-      min_length((uint16_t) min_length_arg),
-      max_length((uint16_t) max_length_arg),
-      flag((uint16_t) flag_arg),
-      min_keypart_map(min_keypart_map_arg),
-      max_keypart_map(max_keypart_map_arg)
-    {}
-};
-
 /**
   Quick select interface.
   This class is a parent for all QUICK_*_SELECT classes.
@@ -285,6 +255,8 @@
 };
 
 struct st_qsel_param;
+class QuickRange;
+class QuickRangeSelect;
 
 /**
  * MRR range sequence, array<QuickRange> implementation: sequence traversal
@@ -301,219 +273,6 @@
 
 uint32_t quick_range_seq_next(range_seq_t rseq, KEY_MULTI_RANGE *range);
 
-/**
- * Quick select that does a range scan on a single key. 
- *
- * The records are returned in key order.
- * 
- */
-class QuickRangeSelect : public QuickSelectInterface
-{
-protected:
-  Cursor *cursor;
-  DYNAMIC_ARRAY ranges; /**< ordered array of range ptrs */
-
-  /** Members to deal with case when this quick select is a ROR-merged scan */
-  bool in_ror_merged_scan;
-  MyBitmap column_bitmap;
-  MyBitmap *save_read_set;
-  MyBitmap *save_write_set;
-  bool free_file; /**< True when this->file is "owned" by this quick select */
-
-  /* Range pointers to be used when not using MRR interface */
-  QuickRange **cur_range; /**< current element in ranges  */
-  QuickRange *last_range;
-
-  /** Members needed to use the MRR interface */
-  QuickRangeSequenceContext qr_traversal_ctx;
-  uint32_t mrr_buf_size; /**< copy from session->variables.read_rnd_buff_size */
-  HANDLER_BUFFER *mrr_buf_desc; /**< the Cursor buffer */
-
-  /** Info about index we're scanning */
-  KEY_PART *key_parts;
-  KEY_PART_INFO *key_part_info;
-
-  bool dont_free; /**< Used by QUICK_SELECT_DESC */
-
-  int cmp_next(QuickRange *range);
-  int cmp_prev(QuickRange *range);
-  bool row_in_ranges();
-public:
-  uint32_t mrr_flags; /**< Flags to be used with MRR interface */
-  MEM_ROOT alloc;
-
-  QuickRangeSelect(Session *session,
-                     Table *table,
-                     uint32_t index_arg,
-                     bool no_alloc,
-                     MEM_ROOT *parent_alloc,
-                     bool *create_err);
-  ~QuickRangeSelect();
-
-  int init();
-  int reset(void);
-  int get_next();
-  void range_end();
-  int get_next_prefix(uint32_t prefix_length,
-                      key_part_map keypart_map,
-                      unsigned char *cur_prefix);
-  bool reverse_sorted()
-  {
-    return false;
-  }
-  bool unique_key_range();
-  int init_ror_merged_scan(bool reuse_handler);
-  void save_last_pos();
-  int get_type()
-  {
-    return QS_TYPE_RANGE;
-  }
-  void add_keys_and_lengths(String *key_names, String *used_lengths);
-  void add_info_string(String *str);
-  void resetCursor()
-  {
-    cursor= NULL;
-  }
-private:
-  /* Used only by QUICK_SELECT_DESC */
-  QuickRangeSelect(const QuickRangeSelect& org) : QuickSelectInterface()
-  {
-    memmove(this, &org, sizeof(*this));
-    /*
-      Use default MRR implementation for reverse scans. No table engine
-      currently can do an MRR scan with output in reverse index order.
-    */
-    mrr_buf_desc= NULL;
-    mrr_flags|= HA_MRR_USE_DEFAULT_IMPL;
-    mrr_buf_size= 0;
-  }
-  friend class ::TRP_ROR_INTERSECT; 
-  friend
-  QuickRangeSelect *get_quick_select_for_ref(Session *session, Table *table,
-                                               struct table_reference_st *ref,
-                                               ha_rows records);
-  friend bool get_quick_keys(PARAM *param, QuickRangeSelect *quick,
-                             KEY_PART *key, SEL_ARG *key_tree,
-                             unsigned char *min_key, uint32_t min_key_flag,
-                             unsigned char *max_key, uint32_t max_key_flag);
-  friend QuickRangeSelect *get_quick_select(PARAM*,uint32_t idx,
-                                              SEL_ARG *key_tree,
-                                              uint32_t mrr_flags,
-                                              uint32_t mrr_buf_size,
-                                              MEM_ROOT *alloc);
-  friend class QUICK_SELECT_DESC;
-  friend class QUICK_INDEX_MERGE_SELECT;
-  friend class QUICK_ROR_INTERSECT_SELECT;
-  friend class QUICK_GROUP_MIN_MAX_SELECT;
-  friend uint32_t quick_range_seq_next(range_seq_t rseq, KEY_MULTI_RANGE *range);
-  friend range_seq_t quick_range_seq_init(void *init_param,
-                                          uint32_t n_ranges, uint32_t flags);
-  friend void select_describe(JOIN *join, bool need_tmp_table, bool need_order,
-                              bool distinct,const char *message);
-};
-
-/**
-  QUICK_INDEX_MERGE_SELECT - index_merge access method quick select.
-
-    QUICK_INDEX_MERGE_SELECT uses
-     * QuickRangeSelects to get rows
-     * Unique class to remove duplicate rows
-
-  INDEX MERGE OPTIMIZER
-    Current implementation doesn't detect all cases where index_merge could
-    be used, in particular:
-     * index_merge will never be used if range scan is possible (even if
-       range scan is more expensive)
-
-     * index_merge+'using index' is not supported (this the consequence of
-       the above restriction)
-
-     * If WHERE part contains complex nested AND and OR conditions, some ways
-       to retrieve rows using index_merge will not be considered. The choice
-       of read plan may depend on the order of conjuncts/disjuncts in WHERE
-       part of the query, see comments near imerge_list_or_list and
-       SEL_IMERGE::or_sel_tree_with_checks functions for details.
-
-     * There is no "index_merge_ref" method (but index_merge on non-first
-       table in join is possible with 'range checked for each record').
-
-    See comments around SEL_IMERGE class and test_quick_select for more
-    details.
-
-  ROW RETRIEVAL ALGORITHM
-
-    index_merge uses Unique class for duplicates removal.  index_merge takes
-    advantage of Clustered Primary Key (CPK) if the table has one.
-    The index_merge algorithm consists of two phases:
-
-    Phase 1 (implemented in QUICK_INDEX_MERGE_SELECT::prepare_unique):
-    prepare()
-    {
-      activate 'index only';
-      while(retrieve next row for non-CPK scan)
-      {
-        if (there is a CPK scan and row will be retrieved by it)
-          skip this row;
-        else
-          put its rowid into Unique;
-      }
-      deactivate 'index only';
-    }
-
-    Phase 2 (implemented as sequence of QUICK_INDEX_MERGE_SELECT::get_next
-    calls):
-
-    fetch()
-    {
-      retrieve all rows from row pointers stored in Unique;
-      free Unique;
-      retrieve all rows for CPK scan;
-    }
-*/
-class QUICK_INDEX_MERGE_SELECT : public QuickSelectInterface
-{
-public:
-  QUICK_INDEX_MERGE_SELECT(Session *session, Table *table);
-  ~QUICK_INDEX_MERGE_SELECT();
-
-  int init();
-  int reset(void);
-  int get_next();
-  bool reverse_sorted()
-  {
-    return false;
-  }
-  bool unique_key_range()
-  {
-    return false;
-  }
-  int get_type()
-  {
-    return QS_TYPE_INDEX_MERGE;
-  }
-  void add_keys_and_lengths(String *key_names, String *used_lengths);
-  void add_info_string(String *str);
-  bool is_keys_used(const MyBitmap *fields);
-
-  bool push_quick_back(QuickRangeSelect *quick_sel_range);
-
-  /* range quick selects this index_merge read consists of */
-  List<QuickRangeSelect> quick_selects;
-
-  /* quick select that uses clustered primary key (NULL if none) */
-  QuickRangeSelect* pk_quick_select;
-
-  /* true if this select is currently doing a clustered PK scan */
-  bool  doing_pk_scan;
-
-  MEM_ROOT alloc;
-  Session *session;
-  int read_keys_and_merge();
-
-  /* used to get rows collected in Unique */
-  READ_RECORD read_record;
-};
-
 
 /**
   Rowid-Ordered Retrieval (ROR) index intersection quick select.
@@ -746,21 +505,6 @@
   void add_keys_and_lengths(String *key_names, String *used_lengths);
 };
 
-class QUICK_SELECT_DESC: public QuickRangeSelect
-{
-public:
-  QUICK_SELECT_DESC(QuickRangeSelect *q, uint32_t used_key_parts,
-                    bool *create_err);
-  int get_next();
-  bool reverse_sorted() { return 1; }
-  int get_type() { return QS_TYPE_RANGE_DESC; }
-private:
-  bool range_reads_after_key(QuickRange *range);
-  int reset(void) { rev_it.rewind(); return QuickRangeSelect::reset(); }
-  List<QuickRange> rev_ranges;
-  List_iterator<QuickRange> rev_it;
-};
-
 /**
  * Executor class for SELECT statements.
  *

=== modified file 'drizzled/optimizer/sum.cc'
--- drizzled/optimizer/sum.cc	2009-12-05 20:06:30 +0000
+++ drizzled/optimizer/sum.cc	2009-12-07 21:50:09 +0000
@@ -74,7 +74,7 @@
                            uint32_t range_fl, 
                            uint32_t prefix_len);
 
-static int maxmin_in_range(bool max_fl, Field* field, COND *cond);
+static int maxmin_in_range(bool max_fl, Field *field, COND *cond);
 
 
 /*
@@ -92,7 +92,6 @@
     UINT64_MAX	Error: Could not calculate number of rows
     #			Multiplication of number of rows in all tables
 */
-
 static uint64_t get_exact_record_count(TableList *tables)
 {
   uint64_t count= 1;
@@ -121,7 +120,7 @@
   table_map outer_tables= 0;
   table_map used_tables= 0;
   table_map where_tables= 0;
-  Item *item;
+  Item *item= NULL;
   int error;
 
   if (conds)
@@ -135,8 +134,8 @@
    */
   for (TableList *tl= tables; tl; tl= tl->next_leaf)
   {
-    TableList *embedded;
-    for (embedded= tl ; embedded; embedded= embedded->embedding)
+    TableList *embedded= NULL;
+    for (embedded= tl; embedded; embedded= embedded->embedding)
     {
       if (embedded->on_expr)
         break;
@@ -285,6 +284,7 @@
                    nullable column, test if there is an exact match for the key.
                  */
                 if (! (range_fl & NEAR_MIN))
+                {
                   /*
                      Closed interval: Either The MIN argument is non-nullable, or
                      we have a >= predicate for the MIN argument.
@@ -293,6 +293,7 @@
                                                        ref.key_buff,
                                                        make_prev_keypart_map(ref.key_parts),
                                                        HA_READ_KEY_OR_NEXT);
+                }
                 else
                 {
                   /*
@@ -530,11 +531,12 @@
 }
 
 
-bool optimizer::simple_pred(Item_func *func_item, Item **args, bool *inv_order)
+bool optimizer::simple_pred(Item_func *func_item, Item **args, bool &inv_order)
 {
   Item *item= NULL;
-  *inv_order= 0;
-  switch (func_item->argument_count()) {
+  inv_order= false;
+  switch (func_item->argument_count()) 
+  {
   case 0:
     /* MULT_EQUAL_FUNC */
     {
@@ -582,7 +584,7 @@
         return 0;
       }
       args[0]= item;
-      *inv_order= 1;
+      inv_order= true;
     }
     else
     {
@@ -643,7 +645,6 @@
   @retval
     1        We can use index to get MIN/MAX value
 */
-
 static bool matching_cond(bool max_fl, 
                           table_reference_st *ref, 
                           KEY *keyinfo,
@@ -735,7 +736,7 @@
   bool inv;
 
   /* Test if this is a comparison of a field and constant */
-  if (! optimizer::simple_pred((Item_func*) cond, args, &inv))
+  if (! optimizer::simple_pred((Item_func*) cond, args, inv))
   {
     return 0;
   }
@@ -894,8 +895,6 @@
     1   Can use key to optimize MIN()/MAX().
     In this case ref, range_fl and prefix_len are updated
 */
-
-
 static bool find_key_for_maxmin(bool max_fl, 
                                 table_reference_st *ref,
                                 Field* field, 

=== modified file 'drizzled/optimizer/sum.h'
--- drizzled/optimizer/sum.h	2009-12-04 07:02:49 +0000
+++ drizzled/optimizer/sum.h	2009-12-07 21:50:09 +0000
@@ -40,7 +40,7 @@
   @retval
     1        Otherwise
 */
-bool simple_pred(Item_func *func_item, Item **args, bool *inv_order);
+bool simple_pred(Item_func *func_item, Item **args, bool &inv_order);
 
 /**
   Substitutes constants for some COUNT(), MIN() and MAX() functions.

=== modified file 'drizzled/sql_select.cc'
--- drizzled/sql_select.cc	2009-12-07 19:18:46 +0000
+++ drizzled/sql_select.cc	2009-12-07 21:50:09 +0000
@@ -59,6 +59,7 @@
 #include "drizzled/optimizer/sargable_param.h"
 #include "drizzled/optimizer/key_use.h"
 #include "drizzled/optimizer/range.h"
+#include "drizzled/optimizer/quick_range_select.h"
 
 using namespace std;
 using namespace drizzled;
@@ -5154,7 +5155,7 @@
       */
       if (! select->quick->reverse_sorted())
       {
-        optimizer::QUICK_SELECT_DESC *tmp= NULL;
+        optimizer::QuickSelectDescending *tmp= NULL;
         bool error= false;
         int quick_type= select->quick->get_type();
         if (quick_type == optimizer::QuickSelectInterface::QS_TYPE_INDEX_MERGE ||
@@ -5164,18 +5165,19 @@
         {
           tab->limit= 0;
           select->quick= save_quick;
-          return(0);                   // Use filesort
+          return 0; // Use filesort
         }
 
         /* order_st BY range_key DESC */
-        tmp= new optimizer::QUICK_SELECT_DESC((optimizer::QuickRangeSelect*)(select->quick),
-                                              used_key_parts, &error);
-        if (!tmp || error)
+        tmp= new optimizer::QuickSelectDescending((optimizer::QuickRangeSelect*)(select->quick),
+                                                  used_key_parts, 
+                                                  &error);
+        if (! tmp || error)
         {
           delete tmp;
-                select->quick= save_quick;
-                tab->limit= 0;
-          return(0);		// Reverse sort not supported
+          select->quick= save_quick;
+          tab->limit= 0;
+          return 0; // Reverse sort not supported
         }
         select->quick=tmp;
       }
@@ -5195,7 +5197,7 @@
   }
   else if (select && select->quick)
     select->quick->sorted= 1;
-  return(1);
+  return 1;
 }
 
 /*

=== modified file 'drizzled/sql_select.h'
--- drizzled/sql_select.h	2009-12-07 19:18:46 +0000
+++ drizzled/sql_select.h	2009-12-07 21:50:09 +0000
@@ -213,9 +213,6 @@
                         drizzled::optimizer::KeyUse *org_keyuse, 
                         table_map used_tables);
 
-/* from sql_delete.cc, used by optimizer/range.cc */
-extern "C" int refpos_order_cmp(void* arg, const void *a,const void *b);
-
 #include "drizzled/stored_key.h"
 
 bool cp_buffer_from_ref(Session *session, table_reference_st *ref);

